@@ -396,6 +396,15 @@
           (recurse (%array-data x))
           (truly-the (integer 128 255) result))))))
 
+(declaim (ftype (sfunction (array) (values (integer 128 255) (unsigned-byte 8)))
+                array-underlying-widetag-and-shift))
+(defun array-underlying-widetag-and-shift (array)
+  (declare (explicit-check))
+  (let ((widetag (array-underlying-widetag array)))
+    (values widetag
+            (truly-the (unsigned-byte 8)
+                       (aref %%simple-array-n-bits-shifts%% widetag)))))
+
 ;; Complain in various ways about wrong MAKE-ARRAY and ADJUST-ARRAY arguments,
 ;; returning the two initialization arguments needed for DATA-VECTOR-FROM-INITS.
 ;; This is an unhygienic macro which would be a MACROLET other than for
@@ -452,10 +461,9 @@
 (defun %save-displaced-new-array-backpointer (array data)
   (flet ((purge (pointers)
            (remove-if-not #'weak-pointer-value pointers)))
-    (when (array-header-p data)
-      (setf (%array-displaced-from data)
-            (cons (make-weak-pointer array)
-                  (purge (%array-displaced-from data)))))))
+    (setf (%array-displaced-from data)
+          (cons (make-weak-pointer array)
+                (purge (%array-displaced-from data))))))
 
 (defmacro populate-dimensions (header list-or-index rank)
   `(if (listp ,list-or-index)
@@ -565,7 +573,8 @@
              (cond (displaced-to
                     (setf (%array-displacement array) (or displaced-index-offset 0))
                     (setf (%array-displaced-p array) t)
-                    (%save-displaced-new-array-backpointer array data))
+                    (when (adjustable-array-p data)
+                      (%save-displaced-new-array-backpointer array data)))
                    (t
                     (setf (%array-displaced-p array) nil)))
              (populate-dimensions array dimensions array-rank)
@@ -693,25 +702,30 @@ of specialized arrays is supported."
                     (make-array ,(1+ widetag-mask)))
                 (declaim (type (simple-array function (,(1+ widetag-mask)))
                                ,table-name))
-                (defmacro ,name (array-var)
-                  `(the function
-                     ;; Assigning TAG to 0 initially produces slightly better
-                     ;; code than would be generated by the more natural expression
-                     ;;   (let ((tag (if (%other-ptr ...) (widetag ...) 0)))
-                     ;; but either way is suboptimal. As expressed, if the array-var
-                     ;; is known to satisfy %other-pointer-p, then it performs a
-                     ;; move-immediate-to-register which is clobbered right away
-                     ;; by a zero-extending load. A peephole pass could eliminate
-                     ;; the first move as effectless.  If expressed the other way,
-                     ;; it would produce a jump around a jump because the compiler
-                     ;; is unwilling to *unconditionally* assign 0 into a register
-                     ;; to begin with. It actually wants to guard an immediate load
-                     ;; when it doesn't need to, as if both consequents of the IF
-                     ;; have side-effects that should not happen.
-                     (let ((tag 0))
-                       (when (%other-pointer-p ,array-var)
-                         (setf tag (%other-pointer-widetag ,array-var)))
-                       (svref ,',table-name tag)))))))
+                (defmacro ,name (array-var &optional vector-check)
+                  (if vector-check
+                      `(the function
+                            (svref ,',table-name (%other-pointer-widetag
+                                                  (locally (declare (optimize (safety 1)))
+                                                    (the vector ,array-var)))))
+                      `(the function
+                            ;; Assigning TAG to 0 initially produces slightly better
+                            ;; code than would be generated by the more natural expression
+                            ;;   (let ((tag (if (%other-ptr ...) (widetag ...) 0)))
+                            ;; but either way is suboptimal. As expressed, if the array-var
+                            ;; is known to satisfy %other-pointer-p, then it performs a
+                            ;; move-immediate-to-register which is clobbered right away
+                            ;; by a zero-extending load. A peephole pass could eliminate
+                            ;; the first move as effectless.  If expressed the other way,
+                            ;; it would produce a jump around a jump because the compiler
+                            ;; is unwilling to *unconditionally* assign 0 into a register
+                            ;; to begin with. It actually wants to guard an immediate load
+                            ;; when it doesn't need to, as if both consequents of the IF
+                            ;; have side-effects that should not happen.
+                            (let ((tag 0))
+                              (when (%other-pointer-p ,array-var)
+                                (setf tag (%other-pointer-widetag ,array-var)))
+                              (svref ,',table-name tag))))))))
   (def !find-data-vector-setter %%data-vector-setters%%)
   (def !find-data-vector-setter/check-bounds %%data-vector-setters/check-bounds%%)
   ;; Used by DO-VECTOR-DATA -- which in turn appears in DOSEQUENCE expansion,
@@ -739,8 +753,8 @@ of specialized arrays is supported."
                ,@decls
                (tagbody ,@forms))))))))
 
-(macrolet ((%ref (accessor-getter extra-params)
-             `(sb-c::%funcall-no-nargs (,accessor-getter array) array index ,@extra-params))
+(macrolet ((%ref (accessor-getter extra-params &optional vector-check)
+             `(sb-c::%funcall-no-nargs (,accessor-getter array ,vector-check) array index ,@extra-params))
            (define (accessor-name slow-accessor-name accessor-getter
                                   extra-params check-bounds)
              `(progn
@@ -755,6 +769,10 @@ of specialized arrays is supported."
                                      ;; dispatch.
                                      (safety 0)))
                   (%ref ,accessor-getter ,extra-params))
+                (defun ,(symbolicate 'vector- accessor-name) (array index ,@extra-params)
+                  (declare (explicit-check)
+                           (optimize speed (safety 0)))
+                  (%ref ,accessor-getter ,extra-params t))
                 (defun ,slow-accessor-name (array index ,@extra-params)
                   (declare (optimize speed (safety 0))
                            (array array))
@@ -881,8 +899,7 @@ of specialized arrays is supported."
        (consp (%array-displaced-p array))))
 
 (declaim (ftype (function (array) nil) invalid-array-error))
-(defun invalid-array-error (array)
-  (declare (optimize allow-non-returning-tail-call))
+(define-error-wrapper invalid-array-error (array)
   (aver (array-header-p array))
   ;; Array invalidation stashes the original dimensions here...
   (let ((dims (%array-displaced-p array))
@@ -896,8 +913,7 @@ of specialized arrays is supported."
 
 (declaim (ftype (function (array t integer &optional t) nil)
                 invalid-array-index-error))
-(defun invalid-array-index-error (array index bound &optional axis)
-  (declare (optimize allow-non-returning-tail-call))
+(define-error-wrapper invalid-array-index-error (array index bound &optional axis)
   (if (invalid-array-p array)
       (invalid-array-error array)
       (error 'invalid-array-index-error
@@ -1045,6 +1061,7 @@ of specialized arrays is supported."
 
 (defun array-element-type (array)
   "Return the type of the elements of the array"
+  (declare (explicit-check array))
   (truly-the (or list symbol)
              (widetag->element-type (array-underlying-widetag array))))
 
@@ -1114,9 +1131,8 @@ of specialized arrays is supported."
   "Return T if the given ARRAY has a fill pointer, or NIL otherwise."
   (array-has-fill-pointer-p array))
 
-(defun fill-pointer-error (vector)
-  (declare (optimize allow-non-returning-tail-call)
-           (optimize (sb-c::verify-arg-count 0)))
+(define-error-wrapper fill-pointer-error (vector)
+  (declare (optimize (sb-c::verify-arg-count 0)))
   (error 'simple-type-error
          :datum vector
          :expected-type '(and vector (satisfies array-has-fill-pointer-p))
@@ -1851,7 +1867,7 @@ function to be removed without further warning."
                                     body))
                             (setf (svref ,table-name ,typecode) #',fun-name))))
        (defmacro ,dispatch-name (&rest args)
-         (check-type (first args) symbol)
+         (aver (symbolp (first args)))
          (let ((tag (gensym "TAG")))
            `(funcall
              (truly-the function
@@ -1902,31 +1918,3 @@ function to be removed without further warning."
 ;;; Horrible kludge for the "static-vectors" system
 ;;; which uses an internal symbol in SB-IMPL.
 (import '%vector-widetag-and-n-bits-shift 'sb-impl)
-
-(defun make-weak-vector (length &key (initial-contents nil contents-p)
-                                     (initial-element nil element-p))
-  (declare (index length))
-  (when (and element-p contents-p)
-    (error "Can't specify both :INITIAL-ELEMENT and :INITIAL-CONTENTS"))
-  ;; Explicitly compute a widetag with the weakness bit ORed in.
-  (let ((type (logior (ash vector-weak-flag array-flags-position) simple-vector-widetag)))
-    ;; These allocation calls are the transforms of MAKE-ARRAY for a vector with
-    ;; the respective initializing keyword arg. This is badly OAOO-violating and
-    ;; almost makes me want to cry, but not quite enough for me to improve it.
-    (if contents-p
-        (let ((contents-length (length initial-contents)))
-          (if (= length contents-length)
-              (replace (truly-the simple-vector
-                                  (allocate-vector #+ubsan nil type length length))
-                       initial-contents)
-              (error "~S has ~D elements, vector length is ~D."
-                     :initial-contents contents-length length)))
-        (fill (truly-the simple-vector
-                         (allocate-vector #+ubsan nil type length length))
-              ;; 0 is the usual default, but NIL makes more sense for weak vectors
-              ;; as it is the value assigned to broken hearts.
-              (if element-p initial-element nil)))))
-
-(defun weak-vector-p (x)
-  (and (simple-vector-p x)
-       (test-header-data-bit x (ash vector-weak-flag array-flags-data-position))))
