@@ -40,8 +40,7 @@
 ;; By happenstance this is the same as small-space-size.
 (defconstant alien-linkage-table-space-size #x100000)
 
-#+gencgc
-;; Define START/END constants for GENCGC spaces.
+;; Define START/END constants for GC spaces.
 ;; Assumptions:
 ;;     We only need very small read-only and static spaces, because
 ;;     gencgc does not purify any more.  We can count on being able to
@@ -55,7 +54,7 @@
 ;;
 ;;     The safepoint page (if enabled) is to be allocated immediately
 ;;     prior to static page.
-(defmacro !gencgc-space-setup
+(defmacro gc-space-setup
     (small-spaces-start
           ;; These keywords variables have to be careful not to overlap with the
           ;; the DEFCONSTANT of the same name, hence the suffix.
@@ -103,7 +102,7 @@
                  appending
                  (let* ((relocatable
                           ;; READONLY is usually movable now.
-                          (member space '(fixedobj text
+                          (member space '(fixedobj text #+relocatable-static-space static
                                           #+immobile-space alien-linkage-table
                                           #-darwin-jit read-only)))
                         (start ptr)
@@ -158,6 +157,8 @@
          (defconstant gencgc-card-table-index-mask
            (1- (ash 1 gencgc-card-table-index-nbits)))))))
 
+(defconstant card-marked #+soft-card-marks 0 #-soft-card-marks 1)
+
 (defconstant-eqx +c-callable-fdefns+
   '(sub-gc
     sb-kernel::post-gc
@@ -197,6 +198,7 @@
     *gc-pending*
     #+sb-safepoint sb-impl::*in-safepoint*
     #+sb-thread *stop-for-gc-pending*
+    sb-impl::*unweakened-vectors*
     *pinned-objects*
     #+gencgc (*gc-pin-code-pages* 0)
     ;; things needed for non-local-exit
@@ -322,6 +324,12 @@
       ;; fdefn pointers in the core. A couple others were high on the list as well.
       hairy-data-vector-set
       hairy-data-vector-ref
+      vector-hairy-data-vector-set
+      vector-hairy-data-vector-ref
+      hairy-data-vector-set/check-bounds
+      hairy-data-vector-ref/check-bounds
+      vector-hairy-data-vector-set/check-bounds
+      vector-hairy-data-vector-ref/check-bounds
       %ldb
       sb-kernel:vector-unsigned-byte-8-p)
   #'equalp)
@@ -354,7 +362,7 @@
   ;; I'm not sure what the constraint on that stems from.
   #+(and x86-64 sb-safepoint) 14 ; the safepoint trap page is at word index -15
   #+(and x86-64 (not sb-safepoint)) 16
-  #+(and (not x86-64) immobile-space) 16
+  #+(and (not x86-64) immobile-space) 14 ; the safepoint trap page is at word index -15
   #+(and (not x86-64) (not immobile-space)) 0)
 
 #+gencgc
