@@ -75,7 +75,7 @@
         ;; Thread-local area, no LOCK needed.
         (with-tls-ea (EA :base tls :base-already-live-p t)
           (inst cmpxchg EA new :maybe-fs))
-        (inst cmp eax no-tls-value-marker-widetag)
+        (inst cmp eax no-tls-value-marker)
         (inst jmp :ne check)
         (move eax old))
       (inst cmpxchg (make-ea :dword :base symbol
@@ -119,7 +119,7 @@
             (done (gen-label)))
         (loadw tls symbol symbol-tls-index-slot other-pointer-lowtag)
         (with-tls-ea (EA :base tls :base-already-live-p t)
-          (inst cmp EA no-tls-value-marker-widetag :maybe-fs)
+          (inst cmp EA no-tls-value-marker :maybe-fs)
           (inst jmp :z global-val)
           (inst mov EA value :maybe-fs))
         (inst jmp done)
@@ -142,7 +142,7 @@
         (loadw value object symbol-tls-index-slot other-pointer-lowtag)
         (with-tls-ea (EA :base value :base-already-live-p t)
           (inst mov value EA :maybe-fs))
-        (inst cmp value no-tls-value-marker-widetag)
+        (inst cmp value no-tls-value-marker)
         (inst jmp :ne check-unbound-label)
         (loadw value object symbol-value-slot other-pointer-lowtag)
         (emit-label check-unbound-label)
@@ -163,7 +163,7 @@
         (loadw value object symbol-tls-index-slot other-pointer-lowtag)
         (with-tls-ea (EA :base value :base-already-live-p t)
           (inst mov value EA :maybe-fs))
-        (inst cmp value no-tls-value-marker-widetag)
+        (inst cmp value no-tls-value-marker)
         (inst jmp :ne ret-lab)
         (loadw value object symbol-value-slot other-pointer-lowtag)
         (emit-label ret-lab)))))
@@ -188,7 +188,7 @@
       (loadw value object symbol-tls-index-slot other-pointer-lowtag)
       (with-tls-ea (EA :base value :base-already-live-p t)
         (inst mov value EA :maybe-fs))
-      (inst cmp value no-tls-value-marker-widetag)
+      (inst cmp value no-tls-value-marker)
       (inst jmp :ne check-unbound-label)
       (loadw value object symbol-value-slot other-pointer-lowtag)
       (emit-label check-unbound-label)
@@ -212,7 +212,7 @@
   (:args (symbol :scs (descriptor-reg)))
   (:results (res :scs (any-reg)))
   (:result-types positive-fixnum)
-  (:args-var args)
+  (:arg-refs args)
   (:generator 2
     ;; The symbol-hash slot of NIL holds NIL because it is also the
     ;; car slot, so we have to strip off the two low bits to make sure
@@ -224,9 +224,6 @@
 
 ;;;; fdefinition (FDEFN) objects
 
-(define-vop (fdefn-fun cell-ref)        ; /pfw - alpha
-  (:variant fdefn-fun-slot other-pointer-lowtag))
-
 (define-vop (safe-fdefn-fun)
   (:translate safe-fdefn-fun)
   (:policy :fast-safe)
@@ -407,7 +404,8 @@
 (define-vop (closure-init)
   (:args (object :scs (descriptor-reg))
          (value :scs (descriptor-reg any-reg)))
-  (:info offset)
+  (:info offset dx)
+  (:ignore dx)
   (:generator 4
     (storew value object (+ closure-info-offset offset) fun-pointer-lowtag)))
 
@@ -419,9 +417,6 @@
 
 ;;;; value cell hackery
 
-(define-vop (value-cell-ref cell-ref)
-  (:variant value-cell-value-slot other-pointer-lowtag))
-
 (define-vop (value-cell-set cell-set)
   (:variant value-cell-value-slot other-pointer-lowtag))
 
@@ -484,7 +479,7 @@
       ;; Compute card mark index and touch the mark byte
       (inst mov card object)
       (inst shr card gencgc-card-shift)
-      (inst and card (make-fixup nil :gc-barrier))
+      (inst and card (make-fixup nil :card-table-index-mask))
       (inst mov (make-ea :byte :base table :index card) 1) ; CARD_MARKED
       ;; set 'written' flag in the code header
       ;; this doesn't need to use :LOCK because the only other writer
