@@ -15,11 +15,12 @@
 #ifndef _GC_PRIVATE_H_
 #define _GC_PRIVATE_H_
 
+#include <stdbool.h>
 #include "genesis/instance.h"
 #include "genesis/funcallable-instance.h"
 #include "genesis/weak-pointer.h"
 #include "immobile-space.h"
-#include "code.h"
+#include "gencgc-internal.h"
 
 #ifdef LISP_FEATURE_GENCGC
 void *collector_alloc_fallback(struct alloc_region*,sword_t,int);
@@ -99,6 +100,7 @@ gc_copy_object_resizing(lispobj object, long nwords, void* region, int page_type
 extern sword_t (*const scavtab[256])(lispobj *where, lispobj object);
 extern struct cons *weak_vectors; /* in gc-common.c */
 extern struct hash_table *weak_hash_tables; /* in gc-common.c */
+extern void acquire_gc_page_table_lock(void), release_gc_page_table_lock(void);
 
 // These next two are prototyped for both GCs
 // but only gencgc will ever call them.
@@ -115,13 +117,13 @@ extern void scan_binding_stack(void);
 extern void scan_finalizers();
 extern void cull_weak_hash_tables(int (*[4])(lispobj,lispobj));
 extern void smash_weak_pointers(void);
-extern boolean scan_weak_hashtable(struct hash_table *hash_table,
+extern bool scan_weak_hashtable(struct hash_table *hash_table,
                                    int (*)(lispobj,lispobj),
                                    void (*)(lispobj*));
 extern int (*weak_ht_alivep_funs[4])(lispobj,lispobj);
 extern void gc_scav_pair(lispobj where[2]);
 extern void gc_common_init();
-extern boolean test_weak_triggers(int (*)(lispobj), void (*)(lispobj));
+extern bool test_weak_triggers(bool (*)(lispobj), void (*)(lispobj));
 
 lispobj  copy_unboxed_object(lispobj object, sword_t nwords);
 
@@ -221,31 +223,6 @@ static inline void assign_generation(lispobj* obj, generation_index_t gen)
 
 #endif /* immobile space */
 
-#ifdef LISP_FEATURE_64_BIT
-#define WEAK_POINTER_CHAIN_END (void*)(intptr_t)1
-#define in_weak_pointer_list(wp) ((wp->header>>16)!=0)
-static inline struct weak_pointer *get_weak_pointer_next(struct weak_pointer *wp) {
-    // 6 bytes to encode 'next' is way more than enough
-    uword_t offset = wp->header >> 16;
-    if (offset <= 1) return (void*)offset;
-    return (void*)((lispobj*)DYNAMIC_SPACE_START + offset);
-}
-static inline void reset_weak_pointer_next(struct weak_pointer *wp) {
-    wp->header = wp->header & 0xffff;
-}
-static inline lispobj encode_weakptr_next(void* x) {
-    if ((uword_t)x <= 1) return (uword_t)x;
-    sword_t wordindex = (lispobj*)x - (lispobj*)DYNAMIC_SPACE_START;
-    gc_assert(wordindex != 0); // wp can't be the first object in dynamic space
-    return wordindex;
-}
-#else
-#define WEAK_POINTER_CHAIN_END (void*)(intptr_t)-1
-#define in_weak_pointer_list(wp) (wp->next)
-#define get_weak_pointer_next(wp) wp->next
-#define reset_weak_pointer_next(wp) wp->next = 0
-#endif
-
 static inline void add_to_weak_pointer_chain(struct weak_pointer *wp) {
     // Better already be fixed in position or we're in trouble
     gc_dcheck(!compacting_p() || !from_space_p(make_lispobj(wp,OTHER_POINTER_LOWTAG)));
@@ -255,11 +232,7 @@ static inline void add_to_weak_pointer_chain(struct weak_pointer *wp) {
      * is denoted by WEAK_POINTER_CHAIN_END which is distinct from NULL.
      * The test of whether the weak pointer has been placed in the chain
      * is performed in 'scav_weak_pointer' */
-#ifdef LISP_FEATURE_64_BIT
-    wp->header = (encode_weakptr_next(weak_pointer_chain)<<16) | (wp->header & 0xffff);
-#else
-    wp->next = weak_pointer_chain;
-#endif
+    set_weak_pointer_next(wp, weak_pointer_chain);
     weak_pointer_chain = wp;
 }
 
@@ -294,7 +267,7 @@ static inline struct bitmap get_layout_bitmap(struct layout* layout)
  * So index 0 may be the layout pointer if #-compact-instance-header,
  * or a user data slot if #+compact-instance-header
  */
-static inline boolean bitmap_logbitp(unsigned int index, struct bitmap bitmap)
+static inline bool bitmap_logbitp(unsigned int index, struct bitmap bitmap)
 {
     unsigned int word_index = index / N_WORD_BITS;
     unsigned int bit_index  = index % N_WORD_BITS;
@@ -302,11 +275,6 @@ static inline boolean bitmap_logbitp(unsigned int index, struct bitmap bitmap)
     return (bitmap.bits[word_index] >> bit_index) & 1;
 }
 
-/* Keep in sync with 'target-hash-table.lisp' */
-#define hashtable_kind(ht) ((ht->flags >> (4+N_FIXNUM_TAG_BITS)) & 3)
-#define hashtable_weakp(ht) (ht->flags & (8<<N_FIXNUM_TAG_BITS))
-#define hashtable_weakness(ht) (ht->flags >> (6+N_FIXNUM_TAG_BITS))
-
 extern unsigned char* gc_card_mark;
 
 #ifdef LISP_FEATURE_DARWIN_JIT
@@ -425,9 +393,6 @@ static inline int instance_length(lispobj header)
 {
     // * Byte 3 of an instance header word holds the immobile gen# and visited bit,
     //   so those have to be masked off.
-    // * fullcgc uses bit index 31 as a mark bit, so that has to
-    //   be cleared. Lisp does not have to clear bit 31 because fullcgc does not
-    //   operate concurrently.
     // * If the object is in hashed-and-moved state and the original instance payload
     //   length was odd (total object length was even), then add 1.
     //   This can be detected by ANDing some bits, bit 10 being the least-significant
@@ -441,7 +406,7 @@ static inline int instance_length(lispobj header)
 // This is index of the bit that differentiates FUNCALLABLE_INSTANCE_WIDETAG
 // from INSTANCE_WIDETAG.
 #define FUNINSTANCE_SELECTOR_BIT_NUMBER 2
-static inline boolean instanceoid_widetag_p(unsigned char widetag) {
+static inline bool instanceoid_widetag_p(unsigned char widetag) {
     return (widetag | (1<<FUNINSTANCE_SELECTOR_BIT_NUMBER)) == FUNCALLABLE_INSTANCE_WIDETAG;
 }
 static inline int instanceoid_length(lispobj header) {
@@ -482,14 +447,13 @@ static inline int layout_depth2_id(struct layout* layout) {
 }
 // Keep in sync with hardwired IDs in CHOOSE-LAYOUT-ID
 // in src/compiler/generic/layout-ids.lisp
-#define WRAPPER_LAYOUT_ID 2
 #define LAYOUT_LAYOUT_ID 3
 #define LFLIST_NODE_LAYOUT_ID 4
 #define BROTHERTREE_UNARY_NODE_LAYOUT_ID 5
 
 /// Return true if 'thing' is a layout.
 /// This predicate is careful, as is it used to verify heap invariants.
-static inline boolean layoutp(lispobj thing)
+static inline bool layoutp(lispobj thing)
 {
     lispobj layout;
     if (lowtag_of(thing) != INSTANCE_POINTER_LOWTAG) return 0;
@@ -497,7 +461,7 @@ static inline boolean layoutp(lispobj thing)
     return layout_depth2_id(LAYOUT(layout)) == LAYOUT_LAYOUT_ID;
 }
 /// Return true if 'thing' is the layout of any subtype of sb-lockless::list-node.
-static inline boolean lockfree_list_node_layout_p(struct layout* layout) {
+static inline bool lockfree_list_node_layout_p(struct layout* layout) {
     return layout_depth2_id(layout) == LFLIST_NODE_LAYOUT_ID;
 }
 
@@ -522,7 +486,7 @@ static const int CONS_PAGE_USABLE_BYTES = MAX_CONSES_PER_PAGE*CONS_SIZE*N_WORD_B
  * given that the words at 'addr' are within range for an allocated page.
  * 'addr' could be a pointer to random data, and this check is merely
  * a heuristic. False positives are possible. */
-static inline boolean plausible_tag_p(lispobj addr)
+static inline bool plausible_tag_p(lispobj addr)
 {
     if (listp(addr))
         return is_cons_half(CONS(addr)->car)
@@ -543,10 +507,16 @@ static inline boolean plausible_tag_p(lispobj addr)
 # define filler_total_nwords(header) ((header)>>N_WIDETAG_BITS)
 #endif
 
-#ifdef LISP_FEATURE_BIG_ENDIAN
-# define assign_widetag(addr, byte) ((unsigned char*)addr)[N_WORD_BYTES-1] = byte
-#else
-# define assign_widetag(addr, byte) *(unsigned char*)addr = byte
-#endif
+extern char * gc_logfile;
+extern void log_generation_stats(char *logfile, char *header);
+extern void print_generation_stats(void);
+extern double generation_average_age(generation_index_t);
+#define PAGE_INDEX_FMT PRIdPTR
+static inline os_vm_size_t npage_bytes(page_index_t npages)
+{
+    gc_assert(npages>=0);
+    return ((os_vm_size_t)npages)*GENCGC_PAGE_BYTES;
+}
+extern os_vm_size_t auto_gc_trigger;
 
 #endif /* _GC_PRIVATE_H_ */
