@@ -13,6 +13,7 @@
 #define _INCLUDED_GLOBALS_H_
 
 #ifndef __ASSEMBLER__
+# include <stdbool.h>
 # include <sys/types.h>
 # include <unistd.h>
 # include "runtime.h"
@@ -23,17 +24,22 @@
 
 #ifndef __ASSEMBLER__
 
+struct lisp_startup_options {
+    char noinform; // read from Lisp by (at least) src/code/cold-init
+};
+extern struct lisp_startup_options lisp_startup_options;
+extern int pre_verify_gen_0; // set and read from Lisp tests
+extern int gencgc_verbose;
+
 extern sword_t next_free_page;
 #define dynamic_space_highwatermark() (next_free_page*GENCGC_PAGE_BYTES+DYNAMIC_SPACE_START)
 
 #ifdef LISP_FEATURE_SB_THREAD
 
 #ifdef LISP_FEATURE_ARM64
-#define foreign_function_call_active_p(thread) \
-    (thread->control_stack_pointer)
+#define foreign_function_call_active_p(thread) (thread->control_stack_pointer)
 #else
-#define foreign_function_call_active_p(thread) \
-    (thread->foreign_function_call_active)
+#define foreign_function_call_active_p(thread) (thread->ffcall_active_p)
 #endif
 
 #else
@@ -45,6 +51,10 @@ extern int foreign_function_call_active;
 extern os_vm_size_t dynamic_space_size;
 extern os_vm_size_t thread_control_stack_size;
 
+#ifdef LISP_FEATURE_RELOCATABLE_STATIC_SPACE
+extern uword_t STATIC_SPACE_START, STATIC_SPACE_END;
+#endif
+
 #ifndef LISP_FEATURE_DARWIN_JIT
 extern uword_t READ_ONLY_SPACE_START, READ_ONLY_SPACE_END;
 #endif
@@ -66,19 +76,11 @@ static inline lispobj points_to_asm_code_p(uword_t ptr) {
     return asm_routines_start <= ptr && ptr < asm_routines_end;
 }
 
-extern boolean alloc_profiling;
 extern os_vm_address_t alloc_profile_buffer;
 extern lispobj alloc_profile_data; // Lisp SIMPLE-VECTOR
 
 extern lispobj arena_chain;
 
-#ifdef LISP_FEATURE_WIN32
-#define ENVIRON _environ
-#else
-#define ENVIRON environ
-#endif
-extern char **ENVIRON;
-
 #if !defined(LISP_FEATURE_SB_THREAD)
 extern lispobj *current_control_stack_pointer;
 #endif
@@ -92,7 +94,7 @@ extern lispobj *current_binding_stack_pointer;
 extern lispobj *read_only_space_free_pointer;
 extern lispobj *static_space_free_pointer;
 
-static inline boolean readonly_space_p(lispobj ptr) {
+static inline bool readonly_space_p(lispobj ptr) {
     return ptr >= READ_ONLY_SPACE_START && (lispobj*)ptr < read_only_space_free_pointer;
 }
 
