@@ -32,49 +32,9 @@ extern int ffs(int);
 #include "genesis/vector.h"
 #include "murmur_hash.h"
 
-#ifdef LISP_FEATURE_USE_SYS_MMAP
-///
-///                **********************************
-///                *  GC MUST NOT ACQUIRE ANY LOCKS *
-///                **********************************
-///
-/// It's normally fine to use the mmap() system call to obtain memory for the
-/// hopscotch tables, unless your C runtime intercepts mmap() and causes it
-/// to sometimes (or always?) need a spinlock. That lock may be owned already,
-/// so GC will patiently wait forever; meanwhile the lock owner is also
-/// waiting forever on GC to finish.
-/// So bypass the C library routine and call the OS directly
-/// in case of a non-signal-safe interceptor such as
-///   https://chromium.googlesource.com/chromium/src/third_party/tcmalloc/chromium/+/refs/heads/master/src/malloc_hook_mmap_linux.h#146
-///
-static inline void* sys_mmap(void* addr, size_t length, int prot, int flags,
-                             int fd, off_t offset) {
-    // "linux-os.h" brings in <syscall.h>, others may need something different.
-    // mmap2 allows large file access with 32-bit off_t. We don't care about that,
-    // but _usually_ only one or the other of the syscalls exists depending on,
-    // various factors. Basing it on word size will pick the right one.
-#ifdef LISP_FEATURE_64_BIT
-    void* result = (void*)syscall(SYS_mmap, addr, length, prot, flags, fd, offset);
-#else
-    void* result = (void*)syscall(SYS_mmap2, addr, length, prot, flags, fd, offset);
-#endif
-    if (result == MAP_FAILED) lose("mmap failed in GC");
-    return result;
-}
-static inline int sys_munmap(void* addr, size_t length) {
-    return syscall(__NR_munmap, addr, length);
-}
-#define hopscotch_allocate(nbytes) \
-  sys_mmap(0, nbytes, PROT_READ|PROT_WRITE, MAP_PRIVATE|MAP_ANONYMOUS, 0, 0)
-#define hopscotch_deallocate(addr,length) sys_munmap(addr, length)
-
-#else
-
 #define hopscotch_allocate(nbytes) os_allocate(nbytes)
 #define hopscotch_deallocate(addr,length) os_deallocate(addr, length)
 
-#endif
-
 typedef struct hopscotch_table* tableptr;
 void hopscotch_integrity_check(tableptr,char*,int);
 
@@ -155,6 +115,20 @@ static sword_t get_val4(tableptr ht, int index) { return ((int32_t*)ht->values)[
 static sword_t get_val2(tableptr ht, int index) { return ((int16_t*)ht->values)[index]; }
 static sword_t get_val1(tableptr ht, int index) { return ((int8_t *)ht->values)[index]; }
 
+#ifdef LISP_FEATURE_SB_SAFEPOINT
+
+// We can safely use malloc + free because there should be no
+// problem of holding a malloc lock from another thread.
+#include <stdlib.h>
+#define cached_allocate(n) calloc(1,n)
+#define cached_deallocate(ptr,size) free(ptr)
+void hopscotch_init() { }
+
+#else
+
+/// We can't safely use malloc because the stop-for-GC signal might be received
+/// in the midst of a malloc while holding a global malloc lock.
+
 /// Hopscotch storage allocation granularity.
 /// Our usual value of "page size" is the GC page size, which is
 /// coarser than necessary (cf {target}/backend-parms.lisp).
@@ -172,7 +146,7 @@ char* cached_alloc[N_CACHED_ALLOCS];
 void hopscotch_init() // Called once on runtime startup, from gc_init().
 {
     // Prefill the cache with 2 entries, each the size of a kernel page.
-    int n_bytes_per_slice = getpagesize();
+    int n_bytes_per_slice = os_reported_page_size;
     int n_bytes_total = N_CACHED_ALLOCS * n_bytes_per_slice;
     char* mem = hopscotch_allocate(n_bytes_total);
     gc_assert(mem);
@@ -249,6 +223,7 @@ static void cached_deallocate(char* mem, uword_t zero_fill_length)
     memset(mem, 0, zero_fill_length);
     cached_alloc[line] = mem;
 }
+#endif
 
 /* Initialize 'ht' for 'size' logical bins with a max hop of 'hop_range'.
  * 'valuesp' makes a hash-map if true; a hash-set if false.
@@ -277,9 +252,13 @@ static void hopscotch_realloc(tableptr ht, int size, char hop_range)
     uword_t storage_size = (sizeof (uword_t) + ht->value_size) * n_keys
         + sizeof (int) * size; // hop bitmasks
 
-    if (ht->keys)
+    if (ht->keys) {
+#ifndef LISP_FEATURE_SB_SAFEPOINT
+        // the usable size is a private-but-visible aspect of the memory block
+        // if not using malloc(). But with malloc we can't really ask the question.
         gc_assert(usable_size(ht->keys) >= storage_size);
-    else
+#endif
+    } else
         ht->keys = (uword_t*)cached_allocate(storage_size);
 
     ht->mem_size  = storage_size;
@@ -301,7 +280,7 @@ uword_t sxhash_simple_string(struct vector* string)
     unsigned int* char_string = (unsigned int*)(string->data);
 #endif
     unsigned char* base_string = (unsigned char*)(string->data);
-    sword_t len = fixnum_value(string->length);
+    sword_t len = vector_len(string);
     uword_t result = 0;
     sword_t i;
     switch (widetag_of(&string->header)) {
@@ -344,7 +323,7 @@ static uword_t vector_sxhash(lispobj* object)
  * and unlike EQUALP because it compares strings case-sensitively.
  * Note: this works on most numbers too, not just vectors.
  */
-static boolean vector_eql(uword_t arg1, uword_t arg2)
+static int vector_eql(uword_t arg1, uword_t arg2)
 {
     if (arg1 == arg2) return 1;
     lispobj* obj1 = (lispobj*)arg1;
@@ -355,7 +334,10 @@ static boolean vector_eql(uword_t arg1, uword_t arg2)
 
     int widetag1 = header_widetag(header1);
     sword_t nwords = sizetab[widetag1](obj1);
-    if (widetag1 < SIMPLE_ARRAY_UNSIGNED_BYTE_2_WIDETAG)
+    // OMGWTF! Widetags have been rearranged so many times, I am not sure
+    // how to keep this code from regressing.
+    // ASSUMPTION: the range of number widetags ends with (COMPLEX DOUBLE-FLOAT)
+    if (widetag1 <= COMPLEX_DOUBLE_FLOAT_WIDETAG)
         // All words must match exactly. Start by comparing the length
         // (as encoded in the header) since we don't yet know that obj2
         // occupies the correct number of words.
@@ -364,8 +346,14 @@ static boolean vector_eql(uword_t arg1, uword_t arg2)
 
     // Vector elements must have already been coalesced
     // when comparing simple-vectors for similarity.
-    return (obj1[1] == obj2[1]) // same length vectors
-        && !memcmp(obj1 + 2, obj2 + 2, (nwords-2) << WORD_SHIFT);
+    // Note that only vectors marked "shareable" will get here, so no
+    // hash-table storage vectors or anything of that nature.
+    struct vector *v1 = (void*)obj1;
+    struct vector *v2 = (void*)obj2;
+    return (vector_len(v1) == vector_len(v2)) // same length vectors
+        && !memcmp(v1->data, v2->data,
+                   // ASSUMPTION: exactly 2 non-data words
+                   (nwords-2) << WORD_SHIFT);
 }
 
 /* Initialize 'ht' for first use, which entails zeroing the counters
@@ -802,7 +790,7 @@ found0:
 
 #undef probe
 
-boolean hopscotch_delete(tableptr ht, uword_t key)
+int hopscotch_delete(tableptr ht, uword_t key)
 {
     gc_dcheck(key);
     int logical_index = hash(ht, key) & ht->mask;
