@@ -28,6 +28,7 @@
 #include "arch.h"
 #include "interr.h"
 #include "immobile-space.h"
+#include "code.h"
 #if defined(LISP_FEATURE_OS_PROVIDES_DLOPEN) && !defined(LISP_FEATURE_WIN32)
 # include <dlfcn.h>
 #endif
@@ -73,8 +74,8 @@ os_zero(os_vm_address_t addr, os_vm_size_t length)
         /* Now deallocate and allocate the block so that it faults in
          * zero-filled. */
 
-        os_invalidate(block_start, block_size);
-        addr = os_validate(NOT_MOVABLE, block_start, block_size, 0, 0);
+        os_deallocate(block_start, block_size);
+        addr = os_alloc_gc_space(0, NOT_MOVABLE, block_start, block_size);
 
         if (addr == NULL || addr != block_start)
             lose("os_zero: block moved! %p ==> %p", block_start, addr);
@@ -96,13 +97,17 @@ void os_deallocate(os_vm_address_t addr, os_vm_size_t len) {
 os_vm_address_t
 os_allocate(os_vm_size_t len)
 {
-    return os_validate(MOVABLE, (os_vm_address_t)NULL, len, 0, 0);
+    return os_alloc_gc_space(0, MOVABLE, (os_vm_address_t)NULL, len);
 }
 
 void
 os_deallocate(os_vm_address_t addr, os_vm_size_t len)
 {
-    os_invalidate(addr,len);
+#ifdef LISP_FEATURE_WIN32
+    gc_assert(VirtualFree(addr, 0, MEM_RELEASE));
+#else
+    if (munmap(addr, len) == -1) perror("munmap");
+#endif
 }
 #endif
 
@@ -118,25 +123,21 @@ os_sem_init(os_sem_t *sem, unsigned int value)
 {
     if (-1==sem_init(sem, 0, value))
         lose("os_sem_init(%p, %u): %s", sem, value, strerror(errno));
-    FSHOW((stderr, "os_sem_init(%p, %u)\n", sem, value));
 }
 
 void
-os_sem_wait(os_sem_t *sem, char *what)
+os_sem_wait(os_sem_t *sem)
 {
-    FSHOW((stderr, "%s: os_sem_wait(%p) ...\n", what, sem));
     while (-1 == sem_wait(sem))
         if (EINTR!=errno)
-            lose("%s: os_sem_wait(%p): %s", what, sem, strerror(errno));
-    FSHOW((stderr, "%s: os_sem_wait(%p) => ok\n", what, sem));
+            lose("os_sem_wait(%p): %s", sem, strerror(errno));
 }
 
 void
-os_sem_post(sem_t *sem, char *what)
+os_sem_post(sem_t *sem)
 {
     if (-1 == sem_post(sem))
-        lose("%s: os_sem_post(%p): %s", what, sem, strerror(errno));
-    FSHOW((stderr, "%s: os_sem_post(%p)\n", what, sem));
+        lose("os_sem_post(%p): %s", sem, strerror(errno));
 }
 
 void
@@ -167,36 +168,28 @@ os_dlsym_default(char *name)
 }
 #endif
 
-int lisp_linkage_table_n_prelinked;
+int alien_linkage_table_n_prelinked;
 void os_link_runtime()
 {
-    int entry_index = 0;
-    lispobj symbol_name;
-    char *namechars;
-    boolean datap;
-    void* result;
-    int j;
-
-    if (lisp_linkage_table_n_prelinked)
+    if (alien_linkage_table_n_prelinked)
         return; // Linkage was already performed by coreparse
 
     struct vector* symbols = VECTOR(SymbolValue(REQUIRED_FOREIGN_SYMBOLS,0));
-    lisp_linkage_table_n_prelinked = vector_len(symbols);
-    for (j = 0 ; j < lisp_linkage_table_n_prelinked ; ++j)
+    alien_linkage_table_n_prelinked = vector_len(symbols);
+    int j;
+    for (j = 0 ; j < alien_linkage_table_n_prelinked ; ++j)
     {
         lispobj item = symbols->data[j];
-        datap = listp(item);
-        symbol_name = datap ? CONS(item)->car : item;
-        namechars = (void*)(intptr_t)(VECTOR(symbol_name)->data);
-        result = os_dlsym_default(namechars);
+        bool datap = listp(item);
+        lispobj symbol_name = datap ? CONS(item)->car : item;
+        char *namechars = vector_sap(symbol_name);
+        void* result = os_dlsym_default(namechars);
 
         if (result) {
-            arch_write_linkage_table_entry(entry_index, result, datap);
+            arch_write_linkage_table_entry(j, result, datap);
         } else { // startup might or might not work. ymmv
             fprintf(stderr, "Missing required foreign symbol '%s'\n", namechars);
         }
-
-        ++entry_index;
     }
 }
 
@@ -204,8 +197,7 @@ void os_unlink_runtime()
 {
 }
 
-boolean
-gc_managed_heap_space_p(lispobj addr)
+bool gc_managed_heap_space_p(lispobj addr)
 {
     if ((READ_ONLY_SPACE_START <= addr && addr < READ_ONLY_SPACE_END)
         || (STATIC_SPACE_START <= addr && addr < STATIC_SPACE_END)
@@ -235,7 +227,7 @@ gc_managed_heap_space_p(lispobj addr)
 /* Remap a part of an already existing memory mapping from a file,
  * and/or create a new mapping as need be */
 void* load_core_bytes(int fd, os_vm_offset_t offset, os_vm_address_t addr, os_vm_size_t len,
-                      int __attribute__((unused)) execute)
+                      int is_readonly_space)
 {
 #if defined LISP_FEATURE_MIPS
     /* Of the few MIPS machines I have access to, one definitely exhibits a
@@ -265,31 +257,40 @@ void* load_core_bytes(int fd, os_vm_offset_t offset, os_vm_address_t addr, os_vm
 #endif
     int fail = 0;
     os_vm_address_t actual;
+    int protection = 0, sharing = MAP_PRIVATE;
+
+#ifdef LISP_FEATURE_DARWIN_JIT
+    protection = OS_VM_PROT_READ | (is_readonly_space ?  OS_VM_PROT_EXECUTE : OS_VM_PROT_WRITE);
+#else
+    /* If mapping to an OS-chosen address, then the assumption is that we're not going to
+     * execute nor write at the mapped address. (Because why would we ? The spaces from
+     * the core have a chosen address at this point) However, the addr=0 case is for
+     * 'editcore' which unfortunately _does_ write the memory. I'd prefer that it not,
+     * but that's not the concern here. */
+    protection = (addr ? (is_readonly_space ? OS_VM_PROT_READ : OS_VM_PROT_ALL)
+                  : OS_VM_PROT_READ | OS_VM_PROT_WRITE);
+    if (is_readonly_space) sharing = MAP_SHARED;
+#endif
+
 #ifdef LISP_FEATURE_64_BIT
-    actual = sbcl_mmap(addr, len,
+    actual = sbcl_mmap(
 #else
     /* FIXME: why does using sbcl_mmap cause failure here? I would guess that it can't
      * pass 'offset' correctly if LARGEFILE is mandatory, which it isn't on 64-bit.
      * Deadlock should be impossible this early in core loading, I suppose, hence
      * on one hand I don't care; but on the other, it would be nice to not to see
      * any use of a potentially hooked mmap() API within this file. */
-    actual = mmap(addr, len,
-#endif
-                  // If mapping to a random address, then the assumption is
-                  // that we're not going to execute the core; nor should we write to it.
-                  // However, the addr=0 case is for 'editcore' which unfortunately _does_
-                  // write the memory. I'd prefer that it not,
-                  // but that's not the concern here.
-#ifdef LISP_FEATURE_DARWIN_JIT
-                  OS_VM_PROT_READ | (execute ?  OS_VM_PROT_EXECUTE : OS_VM_PROT_WRITE),
-#else
-                  addr ? OS_VM_PROT_ALL : OS_VM_PROT_READ | OS_VM_PROT_WRITE,
+     actual = mmap(
 #endif
+                  addr, len, protection,
                   // Do not pass MAP_FIXED with addr of 0, because most OSes disallow that.
-                  MAP_PRIVATE | (addr ? MAP_FIXED : 0),
+                  sharing | (addr ? MAP_FIXED : 0),
                   fd, (off_t) offset);
     if (actual == MAP_FAILED) {
-        perror("mmap");
+        if (errno == ENOMEM)
+            fprintf(stderr, "load_core_bytes: mmap(%p,%zu,...) failed with ENOMEM\n", addr, len);
+        else
+            perror("mmap");
         fail = 1;
     } else if (addr && (addr != actual)) {
         fail = 1;
@@ -327,8 +328,7 @@ void* load_core_bytes_jit(int fd, os_vm_offset_t offset, os_vm_address_t addr, o
 
 #endif
 
-boolean
-gc_managed_addr_p(lispobj addr)
+bool gc_managed_addr_p(lispobj addr)
 {
     struct thread *th;
 
@@ -395,3 +395,55 @@ os_protect(os_vm_address_t address, os_vm_size_t length, os_vm_prot_t prot)
     }
 }
 #endif
+
+lispobj* duplicate_codeblob_offheap(lispobj code)
+{
+    int nwords = code_total_nwords((struct code*)(code-OTHER_POINTER_LOWTAG));
+    lispobj* mem = malloc((nwords+1) * N_WORD_BYTES);
+    if ((uword_t)mem & LOWTAG_MASK) lose("this is unexpected\n");
+    // add 1 word if not dualword aligned
+    lispobj* copy = (lispobj*)((uword_t)mem + ((uword_t)mem & N_WORD_BYTES));
+    memcpy(copy, (char*)code-OTHER_POINTER_LOWTAG, nwords<<WORD_SHIFT);
+    return mem;
+}
+
+#if 0 // interceptors for debugging so I don't have to reinvent them every time
+static void decode_protbits(int prot, char result[4]) {
+    result[0] = (prot & PROT_READ) ? 'r' : '-';
+    result[1] = (prot & PROT_WRITE) ? 'w' : '-';
+    result[2] = (prot & PROT_EXEC) ? 'x' : '-';
+    result[3] = 0;
+}
+static void decode_flagbits(int flags, char result[40]) {
+    char *p = result;
+    char delim = '{';
+#define APPEND(str) { *p++ = delim; delim = '|'; strcpy(p, str); p += sizeof str-1; }
+    if (flags & MAP_PRIVATE) APPEND("Pvt");
+    if (flags & MAP_ANON) APPEND("Anon");
+    if (flags & MAP_NORESERVE) APPEND("NoRsv");
+    if (flags & MAP_JIT) APPEND("JIT");
+#undef APPEND
+    strcpy(p, "}");
+}
+void* sbcl_mmap(void* addr, size_t length, int prot, int flags, int fd, off_t offset) {
+    char decoded_prot[4], decoded_flags[40];
+    decode_protbits(prot, decoded_prot);
+    decode_flagbits(flags, decoded_flags);
+    void* result = mmap(addr, length, prot, flags, fd, offset);
+    fprintf(stderr, "mmap(%p,%lx,%s,%s,%d,%llx)=%p\n", addr, length,
+            decoded_prot, decoded_flags, fd, offset, result);
+    return result;
+}
+int sbcl_munmap(void* addr, size_t length) {
+    int result = munmap(addr, length);
+    fprintf(stderr, "munmap(%p,%lx)=%d\n", addr, length, result);
+    return result;
+}
+int sbcl_mprotect(void* addr, size_t length, int prot) {
+    char decoded_prot[4];
+    decode_protbits(prot, decoded_prot);
+    int result = mprotect(addr, length, prot);
+    fprintf(stderr, "mprotect(%p,%lx,%s)=%d\n", addr, length, decoded_prot, result);
+    return result;
+}
+#endif
