@@ -40,7 +40,6 @@
 #include "interr.h"
 #include "lispregs.h"
 #include "runtime.h"
-#include "alloc.h"
 #include "genesis/primitive-objects.h"
 #include "dynbind.h"
 
@@ -323,10 +322,10 @@ uint32_t os_get_build_time_shared_libraries(uint32_t excl_maximum,
                                        void** opt_store_handles,
                                        const char *opt_store_names[])
 {
-    void* base = opt_root ? opt_root : (void*)runtime_module_handle;
+    char* base = opt_root ? opt_root : (void*)runtime_module_handle;
     /* base defaults to 0x400000 with GCC/mingw32. If you dereference
      * that location, you'll see 'MZ' bytes */
-    void* base_magic_location =
+    char* base_magic_location =
         base + ((IMAGE_DOS_HEADER*)base)->e_lfanew;
 
     /* dos header provided the offset from `base' to
@@ -346,13 +345,13 @@ uint32_t os_get_build_time_shared_libraries(uint32_t excl_maximum,
          * fortunately, those places are irrelevant to the task at
          * hand. */
 
-        IMAGE_FILE_HEADER* image_file_header = (base_magic_location + 4);
+        IMAGE_FILE_HEADER* image_file_header = (void*)(base_magic_location + 4);
         IMAGE_OPTIONAL_HEADER* image_optional_header =
             (void*)(image_file_header + 1);
         IMAGE_DATA_DIRECTORY* image_import_direntry =
             &image_optional_header->DataDirectory[IMAGE_DIRECTORY_ENTRY_IMPORT];
         IMAGE_IMPORT_DESCRIPTOR* image_import_descriptor =
-            base + image_import_direntry->VirtualAddress;
+            (void*)(base + image_import_direntry->VirtualAddress);
         uint32_t nlibrary, j;
 
         for (nlibrary=0u; nlibrary < excl_maximum
@@ -490,9 +489,9 @@ struct {
     CONDITION_VARIABLE cond_has_data;
     CONDITION_VARIABLE cond_has_client;
     HANDLE thread;
-    boolean initialized;
+    bool initialized;
     HANDLE handle;
-    boolean in_progress;
+    bool in_progress;
 } ttyinput;
 
 #ifdef LISP_FEATURE_64_BIT
@@ -570,7 +569,7 @@ static void resolve_optional_imports()
 
 #undef RESOLVE
 
-intptr_t win32_get_module_handle_by_address(os_vm_address_t addr)
+intptr_t win32_get_module_handle_by_address(void* addr)
 {
     HMODULE result = 0;
     /* So apparently we could use VirtualQuery instead of
@@ -752,8 +751,8 @@ win32_reset_stack_overflow_guard_page() {
      * unwinding from a stack overflow condition without retriggering the guard
      * page. See test (:EXHAUST :WRITE-TO-STACK-ON-UNWIND). */
 #define WIN32_STACK_GUARD_SLACK (2*win32_stack_guarantee + win32_page_size)
-    void *stack_guard_start = CONTROL_STACK_GUARD_PAGE(self);
-    fprintf(stderr, "INFO: Reprotecting control stack guard (0x%p+0x%x)\n",
+    char *stack_guard_start = CONTROL_STACK_GUARD_PAGE(self);
+    fprintf(stderr, "INFO: Reprotecting control stack guard (0x%p+0x%lx)\n",
             stack_guard_start, WIN32_STACK_GUARD_SLACK);
     fflush(stderr);
 
@@ -913,7 +912,7 @@ os_protect(os_vm_address_t address, os_vm_size_t length, os_vm_prot_t prot)
 }
 
 /* A tiny bit of interrupt.c state we want our paws on. */
-extern boolean internal_errors_enabled;
+extern int internal_errors_enabled;
 
 extern void exception_handler_wrapper();
 
@@ -1413,8 +1412,7 @@ socket_input_available(HANDLE socket, long time, long utime)
  * on handle, if no cancellation is requested yet (and return TRUE),
  * otherwise clear thread's I/O cancellation flag and return false.
  */
-static
-boolean io_begin_interruptible(HANDLE handle)
+static bool io_begin_interruptible(HANDLE handle)
 {
     /* No point in doing it unless OS supports cancellation from other
      * threads */
@@ -1552,8 +1550,7 @@ static __stdcall unsigned int tty_read_line_server(LPVOID arg)
     return 0;
 }
 
-static boolean
-tty_maybe_initialize_unlocked(HANDLE handle)
+static bool tty_maybe_initialize_unlocked(HANDLE handle)
 {
     if (!ttyinput.initialized) {
         if (!DuplicateHandle(GetCurrentProcess(),handle,
@@ -1573,10 +1570,9 @@ tty_maybe_initialize_unlocked(HANDLE handle)
     return 1;
 }
 
-boolean
-win32_tty_listen(HANDLE handle)
+bool win32_tty_listen(HANDLE handle)
 {
-    boolean result = 0;
+    bool result = 0;
     INPUT_RECORD ir;
     DWORD nevents;
     EnterCriticalSection(&ttyinput.lock);
@@ -1672,11 +1668,10 @@ unlock:
     return result;
 }
 
-boolean
-win32_maybe_interrupt_io(void* thread)
+bool win32_maybe_interrupt_io(void* thread)
 {
     struct thread *th = thread;
-    boolean done = 0;
+    bool done = 0;
 
 #ifdef LISP_FEATURE_SB_FUTEX
     if (thread_extra_data(th)->waiting_on_address)
